function [LEGS, kepIN2] = constructResonantOrbits(plt, res, legprev, tolINCL, minMax)

% DESCRIPTION :
% this function allows for construction of resonant orbits
% 
% INPUT :
% - plt     : target planet
% - res     : vector such as m = res(1) and n = res(2) of the m:n resonance
% - legprev : previous legs of the MGA transfer
% - tolINCL : tolerance over the resonant orbit inclination 
% 
% OUTPUT : 
% - LEGS : last 10 elements of LEGS define the resonant orbits
% -------------------------------------------------------------------------

if nargin == 3
    tolINCL = deg2rad(1);
    minMax  = 1; % --> save the min(DINCL)
elseif nargin == 4
    minMax = 1;
end

mu            = 132724487690;              % --> gravitational parameter of the Sun
[~, ~, smapl] = planetConstants(plt);      % --> planet semi-major axis
Tpl           = 2*pi*sqrt(smapl^3/mu);     % --> period of the planet
Tsc           = res(1)/res(2)*Tpl;         % --> period of the SC
dt            = Tsc/86400;                 % --> transfer time (of the resonant orbit)
sma           = (mu*(Tsc/(2*pi))^2)^(1/3); % --> semi-major axis of the resonant orbit

% --> extract info from previous leg
rrIN  = legprev(:,end-9:end-7); % --> incoming position vector
vvIN  = legprev(:,end-6:end-4); % --> incoming velocity vector
plIN  = legprev(:,end-3);       % --> incoming planet
tIN   = legprev(:,end-2);       % --> incoming epoch
vInf  = legprev(:,end-1);       % --> incoming infinity velocity

% --> incoming keplerian elements (before the first flyby)
kepIN = car2kep([rrIN, vvIN], mu);

% --> planet state at the two encounters 
[rrga1, vvga1] = EphSS_car(plIN, tIN);
[rrga2, vvga2] = EphSS_car(plIN, tIN + res(1)*Tpl/86400);

phi    = deg2rad(0:5:360);
vscou1 = sqrt(mu*(2/norm(rrga1) - 1/sma)); % --> SC velocity after the first flyby

th = acos((vInf^2 + norm(vvga1)^2 - vscou1^2)/(2*vInf*norm(vvga1)));
if abs(imag(th)) < 1e-3
    th    = real(th);
    th    = wrapTo2Pi(th);
    alpha = pi - th;

    vInfIN2 = zeros(length(phi), 1);
    vvIN2   = zeros(length(phi), 3);
    kepIN2  = zeros(length(phi), 6);

    for indphi = 1:length(phi)
    
        PHI      = phi(indphi);
        vvInfOU1 = vInf.*[cos(alpha), sin(alpha)*cos(PHI), -sin(alpha)*sin(PHI)];
    
        vvers = vvga1./norm(vvga1);
        nvers = cross(rrga1, vvga1)./norm(cross(rrga1, vvga1));
        cvers = cross(vvers, nvers);
        M     = [vvers' nvers' cvers'];
    
        vvInfOU1 = M*vvInfOU1';
        vvInfOU1 = vvInfOU1';
        
        vvInfIN2          = vvInfOU1 + vvga1 - vvga2;
        vvIN2(indphi,:)   = vvInfIN2 + vvga2;
    
        rrscou1           = rrga1;
        vvscou1           = vvInfOU1 + vvga1;
        kepscou1          = car2kep([rrscou1, vvscou1], mu);
        [~, vvf]          = FGKepler_dt(kepscou1, dt*86400, mu);
        vvIN2(indphi,:)   = vvf;
        
        vInfIN2(indphi,:) = norm(vvInfIN2);
        kepIN2(indphi,:)  = car2kep([rrga2, vvIN2(indphi,:)], mu);
        
    end
    DINCL = (-kepIN(:,3) + kepIN2(:,3));
    
    % --> eliminate those exceeding the DELTA_MAX constraint
    MAT_VV      = [ vvIN.*ones(size(DINCL,1),3) vvIN2 vInfIN2 [1:size(DINCL,1)]' DINCL ];
    MAT_VV      = sortrows(MAT_VV, size(MAT_VV,2));
    
    ind_to_save = zeros(size(MAT_VV,1),1);
    delta       = zeros(size(MAT_VV,1),1);
    deltaMax    = zeros(size(MAT_VV,1),1);
    for indm = 1:size(MAT_VV,1)
       
        vvin      = MAT_VV(indm,1:3);
        vvin2     = MAT_VV(indm,4:6);
        rrin2     = rrga2;
        [~, vvou] = FGKepler_dt(car2kep([rrin2, vvin2], mu), -dt*86400, mu);
        
        vvinfin = vvin - vvga1;
        vvinfou = vvou - vvga1;
        [~, delta(indm,:), deltaMax(indm,:)] = find_DV_defect(vvinfin, vvinfou, plIN);             
        
        if delta(indm,:) <= deltaMax(indm,:)
            ind_to_save(indm,:) = indm;
        end
        
    end
    ind_to_save = ind_to_save(~all(ind_to_save == 0, 2),:);
    MAT_VV      = MAT_VV(ind_to_save,:);
    DINCL       = MAT_VV(:, end);
    kepIN2      = kepIN2(MAT_VV(:,8),:);
    
    % --> select those orbit that minimize the DELTA_INCLINATION
    % indxs = find(DINCL <= tolINCL);
    if minMax == 1
        [~,indxs] = min(DINCL);
    else
        [~,indxs] = max(kepIN2(:,3)); % --> maximise the inclination using multiple resonant transfers
    end
    
    vvIN2   = MAT_VV(indxs,4:6);
    vInfIN2 = MAT_VV(indxs,7);
    
    leg  = [rrga2.*ones(size(vvIN2,1),3) vvIN2 plIN.*ones(size(vvIN2,1),1) ...
            (tIN+dt).*ones(size(vvIN2,1),1) vInfIN2 0*ones(size(vvIN2,1),1)];
    LEGS = [legprev.*ones(size(leg,1),size(legprev,2)) leg];

else

    LEGS   = [];
    kepIN2 = [];

end

end