function [SEQSr, SEQSe] = allPossibleSequences(pldep, plarr, IDS, maxNumPlts, res)

% DESCRIPTION : 
% this function computes all the possible sequences towards a given planet.
% This is just a combinatorial problem. No informations about the planets
% are used.
%
% INPUT :
% pldep : ID of the departing planet
% plarr : ID of the arrival planet
% IDS   : IDs of the available planets to flyby
% maxNumPlts : maximum number of planets in the sequence (including the
%              departing and arrival ones, i.e. the EVEMEJ would have a
%              maxNumPlts = 6)
%
% OUTPUT : 
% SEQSr : reduced set of sequences (i.e. you do not have the same departing
%         planet on the first and second position, i.e. no VILT is allowed
%         on the first leg)
% SEQSe : extended set of sequences (i.e. you do have the same departing
%         planet on the first and second position, i.e. VILTs are allowed
%         on the first leg)
%
% -------------------------------------------------------------------------

if isempty(find(IDS == plarr))
    disp('The arrival planet is not in the available ones.');
    SEQSr = []; SEQSe = [];
    return
end

% --> length of the sequence
seq_length = maxNumPlts;

% --> find all possible sequences
M  = permn(IDS, seq_length);

% --> exclude those not departing with the specified pldep
M(M(:,1) ~= pldep,:) = [];

% --> exclude those not arriving at the specified plarr
M(M(:,end) ~= plarr,:) = [];

if plarr > 4
    % --> exclude those legs with plarr in the middle of the sequence
    M  = excludeInTheMiddle(M, plarr);
end

% --> exclude those legs with J/S in the middle
if plarr < 5 && max(IDS) > 4
    ind_to_del = zeros(size(M,1),1);
    for indm = 1:size(M,1)
       seq = M(indm,:);
       [max_pl,pos] = max(seq);
       if max_pl > 4 && pos ~= length(seq)
           ind_to_del(indm,:) = indm;
       end
    end
    ind_to_del = ind_to_del(~all(ind_to_del == 0, 2),:);   % --> eliminate rows all equal to zero
    M(ind_to_del,:) = [];
end

SEQSe = M; % --> extended set of sequences

% --> exclude those legs with pldep in the second position
M(M(:,2) == pldep,:) = [];

SEQSr = M; % --> reduced set of sequences

% --> exclude resonances (i.e. repeated planets) --> only if requested --> THIS WORKS FOR THE REDUCED SET
if nargin == 4 || res == 0 % --> if not specified, resonances are not considered
    ind_to_del = zeros(size(SEQSr,1),1);
    for inds = 1:size(SEQSr,1)
       seq = SEQSr(inds,:);
       diff_s = diff(seq);
       if ~isempty( find(diff_s == 0) )
          ind_to_del(inds,:) = inds;
       end
    end
    ind_to_del = ind_to_del(~all(ind_to_del == 0, 2),:);   % --> eliminate rows all equal to zero
    SEQSr(ind_to_del,:) = [];
end

seq_e = SEQSe(:,2:end);
if nargin == 4 || res == 0
    ind_to_del = zeros(size(seq_e,1),1);
    for inds = 1:size(seq_e,1)
       seq = seq_e(inds,:);
       diff_s = diff(seq);
       if ~isempty( find(diff_s == 0) )
          ind_to_del(inds,:) = inds;
       end
    end
    ind_to_del = ind_to_del(~all(ind_to_del == 0, 2),:);   % --> eliminate rows all equal to zero
    SEQSe(ind_to_del,:) = [];
end


% --> if the arrival planet is J/S and the max. planet in the sequence is
% J/S then check

if plarr == 5 && ~isempty(find(IDS == 6))
    
    % --> exclude those legs with Saturn in the middle of the sequence
    SEQSr = excludeInTheMiddle(SEQSr, 6);
    SEQSe = excludeInTheMiddle(SEQSe, 6);
    
elseif plarr == 6 && ~isempty(find(IDS == 5))
    
    % --> exclude those legs with Jupiter in second position
    SEQSr(SEQSr(:,2) == 5,:) = [];
    SEQSe(SEQSe(:,2) == 5,:) = [];
    
    % --> exclude those legs with successive Jupiter flybys
    SEQSr = excludeSuccessivePlanets(SEQSr, 5);
    SEQSe = excludeSuccessivePlanets(SEQSe, 5);
    
end

end