function [DV,rp,iter]=PowSwingByInv(Vin, Vout)
%
%Usage: [DV,rp,iter] = PowSwingByInv(Vin,Vout,alpha)
%
%Outputs:
%           DV:    Velcity Increment of the Powered SwingBy (non dimensional)
%           rp:    Pericenter radius found.
%           iter:  Number of iteration to converge (-1 if convergence is failed)
%
%Inputs:
%           Vin:   Incoming hyperbolic velocity modulus  (non dimensional)
%           Vout:  Outgoing hyperbolic velocity modulus  (non dimensional)
%           alpha: Angle between Vin and Vout (in rad.)
%
%Comments:  The non dimensional units are R for the length and sqrt(mu/R)
%for the velocity --> gravitational constant is one. R may be choosen
%freely as any relevant length. Magic of the non dimensional forms: if we
%forget about dimension and we call the routine, then DV is returned in the
%same units as the input parameters, and rp has to be multiplied by the
%planet gravitational constant (unit consistent with the velocity input)
%to be transformed in the length.

alpha = acos(dot(Vin, Vout)/(norm(Vin)*norm(Vout)));
Vin   = norm(Vin);
Vout  = norm(Vout);

aIN  = 1/Vin^2;    %semimajor axis of the incoming hyperbola
aOUT = 1/Vout^2;  %semimajor axis of the outcoming hyperbola

%We find the perigee radius with an iteration method based on the gradient
%of the function. Attention has to be payed to the initial point as the
%function is not defined for rp<0. The option here implemented considers
%halfing the perigee radius whenever the gradient pushes the next iteration
%in a non defined zone.
i=0;
maxiter=30;     %maximum number of iteration allowed for the gradient method
rp=1;           %Initial point
err=1;
while ((err>1e-8)&&(i<maxiter))
    i=i+1;
    f=asin(aIN/(aIN+rp))+asin(aOUT/(aOUT+rp))-alpha;
    df=-aIN/sqrt(rp^2+2*aIN*rp)/(aIN+rp)-aOUT/sqrt(rp^2+2*aOUT*rp)/(aOUT+rp);
    rpNew=rp-f/df;
    if (rpNew>0)
        err=abs(rpNew-rp);
        rp=rpNew;
    else
        rp=rp/2;
    end
end

%Evaluation of the DV
DV = abs(sqrt(Vout^2+2/rp)-sqrt(Vin^2+2/rp));

%If the maximum number of iteration is achieved the returned number of
%iteration is -1.
iter=i;
if iter==maxiter
    iter=-1;
end

end
